from typing import List, Optional
from pydantic import BaseModel, Field, model_validator


class FaviconConfig(BaseModel):
    type: str  # "fontawesome", "local"
    name: str  # "camera-retro"


class ThumbnailsConfig(BaseModel):
    height: int


class TemplateConfig(BaseModel):
    url: str
    parameters: Optional[dict] = None


class StorageConfig(BaseModel):
    type: str  # "local", "azure-blob", "s3", ...


class LocalStorageConfig(StorageConfig):
    type: str = "local"
    path: str | None = None  # Path to the local folder, must be same as output path


class AzureBlobStorageConfig(StorageConfig):
    type: str = "azure-blob"
    container: str
    connection_string: str


class OutputConfig(BaseModel):
    path: str


class PhotoConfig(BaseModel):
    source: str | None = None
    filename: str | None = None
    thumbnail: str | None = None

    title: Optional[str] = None


class AlbumConfig(BaseModel):
    title: str
    subtitle: Optional[str] = None
    cover: Optional[str] = None

    source: Optional[str] = None
    photos: Optional[List[PhotoConfig]] = None

    @model_validator(mode="before")
    def check_folder_or_photos(cls, values):
        folder, photos = values.get("source"), values.get("photos")
        if not folder and not photos:
            raise ValueError("Either 'folder' or 'photos' must be provided.")
        return values

    id: Optional[int] = None


class GalleryConfig(BaseModel):
    title: str
    subtitle: Optional[str] = None
    favicon: Optional[FaviconConfig] = None
    template: TemplateConfig
    thumbnail: ThumbnailsConfig = Field(default=ThumbnailsConfig(height=400))
    data_source: LocalStorageConfig | AzureBlobStorageConfig
    data_storage: LocalStorageConfig | AzureBlobStorageConfig
    output: OutputConfig
    albums: List[AlbumConfig] = Field(default_factory=list)


class RendererParameters(BaseModel):
    albums: List[AlbumConfig]
    base_url: str
    thumbnail_height: int
    title: str
    subtitle: Optional[str]
    favicon: Optional[str]
    template_parameters: Optional[dict] = None


class RenderedFile(BaseModel):
    name: str
    content: bytes | str



import os

from ..model import LocalStorageConfig
from .base_provider import BaseSourceDataProvider, BaseStorageProvider


class LocalSourceDataProvider(BaseSourceDataProvider):
    def __init__(self, config: LocalStorageConfig):
        if config.path is None:
            raise ValueError("Local storage path not found.")
        self.path = config.path

    def list_images(self, folder: str) -> list:
        images = []
        for filename in os.listdir(os.path.join(self.path, folder)):
            images.append(filename)
        return images

    def get_image_data(self, directory: str, image_name: str) -> bytes:
        with open(os.path.join(self.path, directory, image_name), "rb") as f:
            return f.read()


class LocalStorageProvider(BaseStorageProvider):
    """Storing images locally in same directory as index.html."""

    def __init__(self, config: LocalStorageConfig):
        if config.path is None:
            raise ValueError("Local storage path not found.")
        self.path = config.path

    def upload_image(self, image_data: bytes, directory, image_name) -> str:
        os.makedirs(os.path.join(self.path, directory), exist_ok=True)
        with open(os.path.join(self.path, directory, image_name), "wb") as f:
            f.write(image_data)
        return os.path.join(directory, image_name).replace("\\", "/")

    def file_exists(self, directory: str, image_name: str) -> str | None:
        file_path = os.path.join(directory, image_name)
        if os.path.exists(os.path.join(self.path, file_path)):
            return file_path.replace("\\", "/")
        return None

    def base_url(self) -> str:
        return ""  # Local storage does not have a base URL, files are stored in the same directory as index.html



import os
from .model import GalleryConfig
import yaml
import re


def load_env_variable_and_escape_slash(value: str) -> str:
    variable_value = os.environ[value]
    variable_value = variable_value.replace("\\", "\\\\")  # escape backslashes
    return variable_value


def load_config(config_path: str) -> GalleryConfig:
    with open(config_path, "r") as f:
        content = f.read()
        try:
            content = re.sub(r"\$\{(\w+)\}", lambda match: load_env_variable_and_escape_slash(match.group(1)), content)
        except KeyError as e:
            print(f"Environment variable {e} not found")
            raise e

        obj = yaml.safe_load(content)
        return GalleryConfig(**obj)



from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README-PIP.md").read_text()

import os

print(os.listdir(this_directory))

setup(
    name="ggallery",
    version="1.0.4",
    description="A tool to generate static HTML photo galleries from various data sources.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Radzivon Chorny",
    author_email="mityy2012@gmail.com",
    url="https://github.com/creeston/ggallery",
    packages=find_packages(),
    include_package_data=True,  # Include non-Python files (e.g., templates)
    install_requires=[
        "azure-storage-blob==12.24.1",
        "Pillow==11.1.0",
        "Jinja2==3.1.5",
        "PyYAML==6.0.2",
        "python-dotenv==1.0.1",
        "pydantic==2.10.6",
    ],
    entry_points={
        "console_scripts": [
            "ggallery=ggallery.__main__:main",
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    project_urls={
        "Documentation": "https://github.com/creeston/ggallery",
        "Source": "https://github.com/creeston/ggallery",
        "Tracker": "https://github.com/creeston/ggallery/issues",
    },
    keywords="static html photo gallery generator",
)
